package com.sitemesh;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;

/**
 * Extention of {@link com.sitemesh.Page} providing access to HTML data.
 *
 * <p>The page is parsed and the <code>&lt;title&gt;</code>, <code>&lt;head&gt;</code>
 * (minus the <code>&lt;title&gt;</code>) and <code>&lt;body&gt;</code> are split
 * into chunks. These can then be used by a {@link com.sitemesh.Decorator}.
 * Properties are also extracted from the HTML.</p>
 *
 * <h2>Page Properties</h2>
 * <p>
 *   When the page is parsed, values from certain tags are added to the properties
 *   to allow easy access to them. The following tags have properties extracted
 *   from them.
 * </p>
 * <ul>
 *   <li>
 *     <b>HTML Tag</b><br>
 *     All attributes of the <code>&lt;html&gt;</code>
 *     tag shall be added as properties.
 *   </li>
 *   <li>
 *     <b>Title Tag</b><br>
 *     The contents of the <code>&lt;title&gt;</code> tag
 *     shall be added as the <code>title</code> property.
 *   </li>
 *   <li>
 *     <b>META Tags</b><br>
 *     All the <code>&lt;meta&gt;</code> tags with
 *     <code>name</code> and <code>content</code> attributes
 *     will be added with the <code>meta</code> prefix.
 *   </li>
 *   <li>
 *     <b>BODY Tag</b><br>
 *     All attributes of the <code>&lt;body&gt;</code> tag
 *     shall be added as properties with the
 *     <code>body</code> prefix.
 *   </li>
 * </ul>
 * <h4>Example</h4>
 * <pre>
 *   <xmp>
 *     <html template="funky">
 *       <head>
 *         <title>My Funky Page</title>
 *         <meta name="description" content="Description of my page.">
 *         <meta name="author" content="Bob">
 *         ...
 *       </head>
 *       <body text="#ff00ff" bgcolor="green">
 *         ...
 *       </body>
 *     </html>
 *   </xmp>
 * template=funky
 * title=My Funky Page
 * meta.description=Description of my page.
 * meta.author=Bob
 * body.text=#ff00ff
 * body.bgcolor=green
 * </pre>
 *
 * @author <a href="mailto:joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.7 $
 */
public interface HTMLPage extends Page {

	/**
	 * Write the contents of the <code>&lt;head&gt;</code> tag.
	 */
	void writeHead( OutputStream out ) throws IOException;

	/**
	 * Write the contents of the <code>&lt;head&gt;</code> tag.
	 */
	void writeHead( Writer out ) throws IOException;

	/**
	 * Write the contents of the <code>&lt;body&gt;</code> tag.
	 */
	void writeBody( OutputStream out ) throws IOException;

	/**
	 * Write the contents of the <code>&lt;body&gt;</code> tag.
	 */
	void writeBody( Writer out ) throws IOException;

	/**
	 * Get the Title of the document
	 */
	String getTitle();

	/**
	 * Get the name of the template suitable for the <code>HTMLPage</code>.
	 */
	String getDecoratorName();

}
